// $Id: Event.h,v 1.13 2002/10/11 02:20:15 leres Exp $
//
// Copyright (c) 1995, 1996, 1997, 1998, 1999, 2002
//      The Regents of the University of California.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that: (1) source code distributions
// retain the above copyright notice and this paragraph in its entirety, (2)
// distributions including binary code include the above copyright notice and
// this paragraph in its entirety in the documentation or other materials
// provided with the distribution, and (3) all advertising materials mentioning
// features or use of this software display the following acknowledgement:
// ``This product includes software developed by the University of California,
// Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
// the University nor the names of its contributors may be used to endorse
// or promote products derived from this software without specific prior
// written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

#ifndef event_h
#define event_h

#include "Expr.h"
#include "Func.h"

class Event : public BroObj {
public:
	Event(Func* arg_func, val_list* arg_args, BroObj* arg_obj = 0)
		{
		func = arg_func;
		args = arg_args;
		obj = arg_obj;
		if ( obj )
			Ref(obj);
		next_event = 0;
		}
	~Event();

	void Dispatch()
		{
		(void) func->Call(args);
		if ( obj )
			// obj->EventDone();
			Unref(obj);
		}

	void SetNext(Event* n)		{ next_event = n; }
	Event* NextEvent() const	{ return next_event; }

	void Describe(ODesc* d) const;

protected:
	Func* func;
	val_list* args;
	BroObj* obj;
	Event* next_event;
};

class EventMgr : public BroObj {
public:
	EventMgr();
	~EventMgr();

	void QueueEvent(Func* f, val_list* vl, BroObj* obj = 0)
		{
		Ref(f);
		QueueEvent(new Event(f, vl, obj));
		}

	void Dispatch();

	void Drain();

	int HasEvents() const	{ return head != 0; }

	void Describe(ODesc* d) const;

protected:
	void QueueEvent(Event* event);

	Event* head;
	Event* tail;
};

extern EventMgr mgr;
extern Func* no_handler;

#endif
